#ifndef CMDLINEPARSER_H
#define CMDLINEPARSER_H

// all options return a string, or null for unset. Boolean options flags) return either "true" or null
// PositionalParameters start at 1!
class CmdlineParser {
public:
	CmdlineParser( int Argc, char *Argv[] );
	~CmdlineParser();

	bool AddOption( int Option, const char *Name, const char *Description, bool WantArg = false, const char *Default = NULL );
	// Max < 0 -> any number of positionals
	bool AddPositionals( unsigned int Min, unsigned int Max, const char *Names, const char *Description );
	bool AddPositionals( unsigned int Num, const char *Names, const char *Description );
	bool Parse();
	SimpleString Usage();

	bool IsValid() { return m_IsParsed; }
	int NumPositionals() { return m_PositionalValues.Size(); }

	const char *operator[]( const char *Name );
	const char *operator[]( int Position );
	const char *ProgramName();
private:
	struct OptionDesc {
		SimpleString m_Option;
		SimpleString m_Name;
		SimpleString m_Description;
		bool m_WantArg;
		bool m_HasDefault;
		SimpleString m_Default;

		OptionDesc( int Option, const char *Name, const char *Description, bool WantArg, const char *Default = NULL )
		: m_Option( SimpleString::PrintF( "-%c", Option ) )
		, m_Name( Name )
		, m_Description( Description )
		, m_WantArg( WantArg )
		, m_HasDefault( Default != NULL )
		, m_Default( Default )
		{}
	};

	bool m_IsParsed;
	Array<SimpleString> m_Argv;
	unsigned int m_MinPositionalArgs,  m_MaxPositionalArgs;
	SimpleString m_PositionalNames, m_PositionalDescription;

	SimpleString m_ProgramName;
	Array<OptionDesc> m_Options;
	Map<SimpleString, SimpleString> m_Values;
	Array<SimpleString> m_PositionalValues;

	bool Error( const char *Message );
	OptionDesc *FindOption( SimpleString Option );
	void SetDefaultValues();
};

#endif // CMDLINEPARSER_H
