#include "core.h"
#include "uimanager.h"
#include "uiscreen-inputdialog.h"
#include "Widgets/uiwidget-text.h"
#include "Widgets/uiwidget-input.h"
#include "configmanager.h"
#if BUILD_WINDOWS
#include "consolemanager.h"
#endif

UIScreenInputDialog::UIScreenInputDialog()
:	m_OKEventName()
,	m_OKCommand()
,	m_OKCallback()
,	m_CancelEventName()
,	m_CancelCommand()
,	m_CancelCallback()
{
	InitializeFromDefinition( "InputDialog" );
}

UIScreenInputDialog::~UIScreenInputDialog() {}

static void InputOKCallback( void *pWidget, void *pUserdata )
{
	(void) pWidget;
	UIScreenInputDialog *pDialog = reinterpret_cast<UIScreenInputDialog*>( pUserdata );
	pDialog->OKCallback();
	STATICHASH( Pop );
	UIScreen::m_UIManager->PostEvent( sPop );
}

static void InputCancelCallback( void *pWidget, void *pUserdata )
{
	(void) pWidget;
	UIScreenInputDialog *pDialog = reinterpret_cast<UIScreenInputDialog*>( pUserdata );
	pDialog->CancelCallback();
	STATICHASH( Pop );
	UIScreen::m_UIManager->PostEvent( sPop );
}

void UIScreenInputDialog::SetParameters(
	bool				PauseGame,
	const SimpleString&	InputString,
	const SimpleString&	InputDynamicString,
	const SimpleString& DefaultString,
	const HashedString&	OKEvent,
	const HashedString&	CancelEvent,
	const SimpleString&	OKCommand,
	const SimpleString&	CancelCommand,
	const SUICallback&	OKCallback,
	const SUICallback&	CancelCallback )
{
	m_PausesGame = PauseGame;

	// HACKY: Referencing widgets directly by index
	UIWidgetText* pDialog = static_cast< UIWidgetText* >( m_RenderWidgets[1] );
	UIWidgetInput* pInput = static_cast< UIWidgetInput* >( m_RenderWidgets[2] );
	UIWidgetText* pOK = static_cast< UIWidgetText* >( m_RenderWidgets[3] );
	UIWidgetText* pCancel = static_cast< UIWidgetText* >( m_RenderWidgets[4] );

	// NOTE: This bypasses the "IsLiteral" property of the widget and always uses localized
	//pDialog->m_String = ConfigManager::GetLocalizedString( InputString, "" );
	//pDialog->m_DynamicString = ConfigManager::GetLocalizedString( InputDynamicString, "" );
	pDialog->m_String = InputString;
	pDialog->m_DynamicString = InputDynamicString;
	pDialog->InitializeFromDefinition( pDialog->m_Name );

	pInput->SetString( DefaultString );
	pInput->InitializeFromDefinition( pInput->m_Name );

	pInput->m_Callback = SUICallback( InputOKCallback, this );
	pOK->m_Callback = SUICallback( InputOKCallback, this );
	pCancel->m_Callback = SUICallback( InputCancelCallback, this );

	m_OKEventName = OKEvent;
	m_OKCommand = OKCommand;
	m_OKCallback = OKCallback;

	m_CancelEventName = CancelEvent;
	m_CancelCommand = CancelCommand;
	m_CancelCallback = CancelCallback;
}

void UIScreenInputDialog::Closed()
{
	CancelCallback();

	if ( m_FocusedWidget )
	{
		m_FocusedWidget->LoseFocus();
	}
}

void UIScreenInputDialog::OKCallback()
{
	if( m_OKEventName != HashedString::NullString )
	{
		m_UIManager->PostEvent( m_OKEventName );
	}

#if BUILD_WINDOWS
	if( m_OKCommand != "" )
	{
		ConsoleManager::GetInstance()->PushCommands( m_OKCommand );
	}
#endif

	if( m_OKCallback.m_Callback )
	{
		m_OKCallback.m_Callback( this, m_OKCallback.m_Void );
	}
}

void UIScreenInputDialog::CancelCallback()
{
	if( m_CancelEventName != HashedString::NullString )
	{
		m_UIManager->PostEvent( m_CancelEventName );
	}

#if BUILD_WINDOWS
	if( m_CancelCommand != "" )
	{
		ConsoleManager::GetInstance()->PushCommands( m_CancelCommand );
	}
#endif

	if( m_CancelCallback.m_Callback )
	{
		m_CancelCallback.m_Callback( this, m_CancelCallback.m_Void );
	}
}

SimpleString UIScreenInputDialog::GetString()
{
	UIWidgetInput* pInput = static_cast< UIWidgetInput* >( m_RenderWidgets[2] );
	return pInput->GetString();
}