#include "core.h"
#include "checksum.h"
#include "filestream.h"
#include "map.h"
#include "cmdlineparser.h"

CmdlineParser ParseCommandline( int Argc, char *Argv[] )
{
	CmdlineParser Opts( Argc, Argv );
	Opts.AddOption( 's', "short", "short output" );
	Opts.AddPositionals( 1, "<infile>", "file to compute checksum of" );
	if ( !Opts.Parse() )
	{
		puts( Opts.Usage().CStr() );
		exit( 1 );
	}
	return Opts;
}

int main( int Argc, char* Argv[] )
{
	CmdlineParser Opts = ParseCommandline( Argc, Argv );

	bool ShortOutput = false;
	uint8* FileContents = NULL;
	int Length = 0;

	if( Opts[ "short" ] )
	{
		ShortOutput = true;
	}
	
	FileStream ChecksumFile( Opts[ 1 ], FileStream::EFM_Read );
	Length = ChecksumFile.Size();
	FileContents = new uint8[ Length ];
	ChecksumFile.Read( Length, FileContents );

	if( FileContents && Length )
	{
		uint32 FileChecksum = Checksum::Adler32( FileContents, Length );
		delete FileContents;

		if( ShortOutput )
		{
			PRINTF( "%s\n", Opts[ 1 ] );
			PRINTF( "%d\n", Length );
			PRINTF( "0x%08X\n", FileChecksum );
		}
		else
		{
			PRINTF( "Filename: %s\n", Opts[ 1 ] );
			PRINTF( "Length:   %d\n", Length );
			PRINTF( "Checksum: 0x%08X\n", FileChecksum );
		}
	}

	return 0;
}